import asyncio
import datetime as dt
import logging
import os

import paho.mqtt.publish as mqtt_publish

from webcontrol.frigate_config import FrigateConfigFile
from webcontrol.nighttime import get_nighttimes

logger = logging.getLogger(__name__)


async def set_camera_detection(camera: str, detection: bool) -> None:
    mqtt_auth = {"username": os.environ.get("MQTT_USERNAME"), "password": os.environ.get("MQTT_PASSWORD")}
    if not all(mqtt_auth.values()):
        mqtt_auth = None

    logger.debug(f"Setting {camera} camera {detection=}")
    mqtt_publish.single(f"frigate/{camera}/detect/set", "ON" if detection else "OFF", hostname="mqtt", port=1883, auth=mqtt_auth)
    logger.info(f"Set {camera} camera {detection=}")


async def reset_all_cameras_detection_after_sunset() -> None:
    frigate_config_file = FrigateConfigFile()
    async for nighttime in get_nighttimes("Winter Haven, FL"):
        logger.info(f"Waiting until {nighttime} to reset detection for all cameras")
        await asyncio.sleep((nighttime - dt.datetime.now()).total_seconds())

        await frigate_config_file.reload()
        active_and_detection_enabled_cameras = set(frigate_config_file.active_cameras).intersection(frigate_config_file.detection_enabled_cameras)
        for camera in active_and_detection_enabled_cameras:
            await set_camera_detection(camera, True)
